// JsCore
// Copyright (C) 2006 AJAXForms S.L.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// Contact the author at: <contact@ajaxforms.com>.

function Calendar() {
    var calendar = this;
    var body = document.getElementsByTagName("BODY")[0];
    this.element = Core.createElement("table", body, null, "calendar");

    var tHead = Core.createElement("thead", this.element);
    var trTitle = Core.createElement("tr", tHead);
    var title = Core.createElement("td", trTitle, null, "title");
    title.colSpan = 7;

    this.selectMonth = Core.createElement("select", title);
    Event.attach(this.selectMonth, "change", function() {
        Calendar.INSTANCE.refresh();
    } );

    for (var i = 0; i < 12; i++) {
        this.selectMonth.options[i] = new Option(I8N.get("calendar.month" + i), i);
    }

    this.inputYear = Core.createElement("input", title);
    this.inputYear.readOnly = true;
    Event.attach(this.inputYear, "mouseup", function() {
        var cal = Calendar.INSTANCE;
        cal.yearList.show();
    } );
    Event.attach(this.inputYear, "change", function() {
        Calendar.INSTANCE.refresh();
    } );

    Event.attach(this.inputYear, "focusout", function() {
        var cal = Calendar.INSTANCE;
        Dialog.show(cal.element,cal.input);
    } );

    var close = Core.createElement("button", title, "X");
    close.setAttribute("title", "Close");

    Event.attach(close, "click", function() {
        Calendar.close();
    } );

    var trDays = Core.createElement("tr", tHead, null, "names");
    var ini = parseInt(I8N.get("calendar.initDay"));

    for (var j = 0; j < 7; j++) {
    	var ind = j + ini;
        this.createElement(trDays, "name", I8N.get("calendar.day" + (ind >= 7? ind - 7 : ind)))
    }

    this.tBody = Core.createElement("tbody", this.element);

    var handler = function(event) {
        var value = Event.getTarget(event).childNodes[0].nodeValue;
        var cal = Calendar.INSTANCE;

        if (value != "") {
	        cal.day = value;
    	    var date = new Date();
        	date.setYear(cal.inputYear.value);
	        date.setMonth(cal.selectMonth.value);
    	    date.setDate(cal.day);
    	    
            if (cal.isTimestamp) {
    	    	date.setMinutes(cal.inputMin.value);
    	    	date.setHours(cal.inputHour.value);
    	    	cal.input.value = I8N.format(date);
            } else {
                cal.input.value = I8N.formatDate(date);
            }

	        Calendar.close();
    	    Event.dispatch(cal.input, "change");
    	    cal.input.focus();
    	}
    };

    for (var i = 0; i < 6; i++) {
        var trLine = Core.createElement("tr", this.tBody);

        for (var j = 0; j < 7; j++) {
            this.createElement(trLine, "day", " ", 1, handler)
        }
    }
    
    var tFoot = Core.createElement("tFoot", this.element);
    var trFoot = Core.createElement("tr", tFoot);
    var tdFoot = Core.createElement("td", trFoot);
   	tdFoot.colSpan = 7;
   	
    this.inputHour = Core.createElement("input", tdFoot);
    this.inputHour.readOnly = true;
    Event.attach(this.inputHour, "mouseup", function() {
        Calendar.INSTANCE.hourList.show();
    } );
    
    tdFoot.appendChild(document.createTextNode(":"));
    this.inputMin = Core.createElement("input", tdFoot);
    this.inputMin.readOnly = true;
    Event.attach(this.inputMin, "mouseup", function() {
        Calendar.INSTANCE.minList.show();
    } );

    tdFoot.appendChild(document.createTextNode(":"));
    this.inputSec = Core.createElement("input", tdFoot);
    this.inputSec.readOnly = true;
    Event.attach(this.inputSec, "mouseup", function() {
    	if (Calendar.INSTANCE.type >= Calendar.SECONDS) {
	        Calendar.INSTANCE.secList.show();
	    }
    } );

    this.yearList = new NumberList(title, "calendarList", this.inputYear, 1900, 2050);
    this.hourList = new NumberList(tdFoot, "calendarList", this.inputHour, 0, 23, 2);
    this.minList = new NumberList(tdFoot, "calendarList", this.inputMin, 0, 59, 2);
    this.secList = new NumberList(tdFoot, "calendarList", this.inputSec, 0, 59, 2);
};

Calendar.prototype.today = function() {
	this.refreshControls(new Date());
};

Calendar.prototype.refreshControls = function(date) {
    this.day = date.getDate();
    this.selectMonth.value = date.getMonth();
    this.inputYear.value = date.getYear() < 1000? 1900 + date.getYear() : date.getYear();

    if (this.isTimestamp) {
	    this.inputHour.value = zeros(date.getHours(), 2);
    	this.inputMin.value = this.type >= Calendar.MINUTES? zeros(date.getMinutes(), 2) : "00";
    	this.inputSec.value = this.type >= Calendar.SECONDS? zeros(date.getSeconds(), 2) : "00";
   	}
   	
   	this.refresh();
};

Calendar.prototype.refresh = function() {
    var firstDay = this.getFirstDay();
    var daysOfMonth = this.getDaysOfMonth();
    var cont = 0;
    var day = 1;
    var currentMonthYear = this.selectMonth.value == this.currentMonth
        && this.inputYear.value == this.currentYear;

    for (var i = 0; i < 6; i++) {
        var trLine = this.tBody.childNodes[i];

        for (var j = 0; j < 7; j++, cont++) {
            var cell = trLine.childNodes[j];
            Core.setClass(cell, "hover", false);
            Core.setClass(cell, "today", currentMonthYear && day == this.currentDay);
            Core.setClass(cell, "selected", day == this.day);
            Core.setClass(cell, "weekend", j > 4);

            cell.firstChild.nodeValue
                = (cont >= firstDay && cont < firstDay + daysOfMonth)? day++ : "";
        }
    }
};

Calendar.prototype.getFirstDay = function() {
   var date = new Date();
   date.setDate(1);
   date.setMonth(this.selectMonth.value);
   date.setYear(this.inputYear.value);

   return date.getDay() == 0? 6 : date.getDay() - 1;
};

Calendar.prototype.getDaysOfMonth = function() {
	var year = this.inputYear.value;
	var month = this.selectMonth.value;

	if (month == 1 && ((0 == (year % 4)) && (   (0 != (year % 100))
	                                         || (0 == (year % 400))))) {
		return 29;
	}

    return Calendar.daysOfMoth[this.selectMonth.value];
};

Calendar.prototype.createElement = function(parent, className, text, colspan, handler) {
    var element = Core.createElement("td", parent, text, className);

    if (colspan > 1) {
        element.colSpan = colspan;
    }
    
    if (handler) {
        Event.attach(element, "click", handler);
        Core.initHover(element);
    }

    return element;
};

Calendar.daysOfMoth = new Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

Calendar.ONLY_DATE = 0;
Calendar.HOURS = 1;
Calendar.MINUTES = 2;
Calendar.SECONDS = 3;

Calendar.show = function(input, type) {
    var cal = Calendar.INSTANCE;

    if (!cal) {
        cal = new Calendar();
        Calendar.INSTANCE = cal;
    }

	if (!type) {
		type = Calendar.ONLY_DATE;
	}

    cal.input = input;
    cal.type = type;
    cal.isTimestamp = type != Calendar.ONLY_DATE;
    Core.setClass(cal.element, "date", !cal.isTimestamp);
    var date;
    
    try {
        date = cal.isTimestamp? I8N.parse(input.value) : I8N.parseDate(input.value);
    } catch (e) { date = new Date(); }

    if (date != null) {
	    cal.refreshControls(date);
    } else {
        cal.today();
    }
    
    Dialog.show(cal.element, input);
};

Calendar.close = function() {
    var cal = Calendar.INSTANCE;
    cal.yearList.close();
    Dialog.hide(cal.element);
};
var Core = {
    fileName : "jsCore.js",
    isOpera : navigator.userAgent.match(/\bOpera\b/) != null,
    isIE : navigator.userAgent.match(/\bMSIE\b/) != null
           && navigator.userAgent.match(/\bOpera\b/) == null,
    isMozilla : navigator.userAgent.match(/\bGecko\b/) != null,
    setClass : function(element, className, value) {
        assert(element && className);

        if (value) {
            if (!this.hasClass(element, className)) {
                element.className += " " + className;
            }
        } else if (element.className) {
            element.className = element.className.replace(className, "");
        }
    },
    hasClass : function(element, className) {
    	var cn = element.className;

    	return inArray(className, (cn && cn.split(" ")) || []);
    },
    initHover : function(element) {
        Event.attach(element, "mouseover", function(event) {
            Core.setClass(Event.getTarget(event), "hover", true);
        } );

        Event.attach(element, "mouseout", function(event) {
            Core.setClass(Event.getTarget(event), "hover", false);
        } );
    },
    getEventPos : function(ev) {
        ev = ev || window.event;
		return { x : ev.pageX || ev.clientX + window.document.body.scrollLeft || 0,
		         y : ev.pageY || ev.clientY + window.document.body.scrollTop || 0 };
    },
    getAbsolutePos : function(e) {
        var r = Core.getPos(e);

        if (e.offsetParent) {
            var tmp = Core.getAbsolutePos(e.offsetParent);
            r.x += tmp.x;
            r.y += tmp.y;
        }

        return r;
    },
    getPos : function(e) {
        var is_div = /^div$/i.test(e.tagName);

        var r = {
            x: e.offsetLeft - (is_div && e.scrollLeft? e.scrollLeft : 0),
            y: e.offsetTop - (is_div && e.scrollTop? e.scrollTop : 0)
        };

        return r;
    },
    setPos : function(element, left, top) {
        if (element.offsetParent) {
            var tmp = Core.getAbsolutePos(element.offsetParent);
            left -= tmp.x;
            top -= tmp.y;
        }

        element.style.top = top + "px";
        element.style.left = left + "px";
    },
    loadProperties : function(name) {
        if (!this.ROOT) {
            var scripts = document.getElementsByTagName("SCRIPT");

            for (var i = 0; i < scripts.length; i++) {
                var src = scripts[i].src;

                if (src.indexOf("calendar2.js") != -1) {
                    this.ROOT = src.replace("calendar2.js", "");
                    break;
                }
            }
        }
        
        var uri = this.ROOT + name + ".properties";
        var req = Core.openRequest("GET", uri, false);

        if (req.overrideMimeType) {
	        req.overrideMimeType("text/plain");
	    }

        try {        
            req.send(null);
        } catch(e) {
            alert("File not found: " + uri);
        }

        var text = req.responseText;
        var lines = text.split(/\s*\n\s*/);
        var properties = [];
        
        for (var i = 0; i < lines.length; i++) {
        	var line = lines[i].replace(/^\s+|\s+$/, ''); //trim

        	if (line[0] != '#' && line[0] != '!') {
	            var spl = lines[i].split(/\s*=\s*/);
	            properties[spl[0]] = spl[1];
	        }
        }
        
        return properties;
    },
    constructURI : function(uri) {
        if (!uri.match(/\/\//)) {
            uri = document.location.href.replace(/[^\/]+$/,"") + uri;
        }
    
        return uri;
    },
    createElement : function(type, parent, content, className) {
        var el = document.createElement(type);
        if (className) el.className = className;
        if (parent) parent.appendChild(el);
        if (content) el.appendChild(document.createTextNode(content));
        return el;
    },
    getWindowSize : function() {
        return {
            height : document.body.clientHeight || document.documentElement.clientHeight || window.innerHeight || 0,
            width : document.body.clientWidth || document.documentElement.clientWidth || window.innerWidth || 0
        };
    }
};

if (Core.isIE) {
    Core.openRequest = function(method, uri, async) {
        var req = new ActiveXObject("Msxml2.XMLHTTP");
        req.open(method, uri, async);

        return req;
    };
} else {
    Core.openRequest = function(method, uri, async) {
    //  netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
        var req = new XMLHttpRequest();
        req.open(method, Core.constructURI(uri), async);
        if (Core.isMozilla) req.overrideMimeType("text/xml");

        return req;
    };
}
var DebugConsole = {
    element_ : null,
    isInit_ : false,
    time_ : 0,
    init_ : function() {
        this.element_ = $("console");
        this.isInit_ = true;
        this.time_ = new Date().getTime();
    },
    write : function(text) {
        if (this.isOpen()) {
            var time = new Date().getTime();
            this.element_.appendChild(document.createTextNode(time - this.time_ + " -> " + text));
            Core.createElement("br", this.element_);
            this.time_ = time;
        }
    },
    clear : function() {
        if (this.isOpen()) {
            while (this.element_.firstChild) {
                this.element_.removeChild(this.element_.firstChild);
            }

            this.time_ = new Date().getTime();
        }
    },
    isOpen : function() {
        if (!this.isInit_) {
            this.init_();
        }
        
        return this.element_ != null;
    }
};var Dialog = {
    dialogs : [],
    init : false,
    show : function(div, parent) {
        if (!this.init) {
            this.init = true;

            Event.attach(document, "mousedown", function(event) {
                var target = Event.getTarget(event);
                var ds = Dialog.dialogs;
                
                for (var i = 0; i < ds.length; i++) {
                    var d = ds[i];

                    if (d.style.display != "none") {
	                    var t = target;
	                    
	                    for (; t && t != d; t = t.parentNode);
	                    
	                    if (!t) Dialog.hide(d);
	                }
                }
            } );
        }

        div = typeof div == "string"? $(div) : div;
        div.style.display = "block";

        if (parent) {
            var absPos = Core.getAbsolutePos(parent);
            Core.setPos(div, absPos.x, (absPos.y + parent.offsetHeight));
        } else {        
     		var size = Core.getWindowSize();
     		var h = (size.height - div.offsetHeight) / 2;
            Core.setPos(div, (size.width - div.offsetWidth) / 2,
                h > 0? h : 100);
        }

        this.showSelects(div, false);
        
        if (!inArray(div, this.dialogs)) {
            this.dialogs.push(div);
        }
    },
    show1 : function(div, parent) {
        if (!this.init) {
            this.init = true;

            Event.attach(document, "mousedown", function(event) {
                var target = Event.getTarget(event);
                var ds = Dialog.dialogs;
                
                for (var i = 0; i < ds.length; i++) {
                    var d = ds[i];

                    if (d.style.display != "none") {
	                    var t = target;
	                    
	                    for (; t && t != d; t = t.parentNode);
	                    
	                    if (!t) Dialog.hide(d);
	                }
                }
            } );
        }

        div = typeof div == "string"? $(div) : div;
        div.style.display = "block";

        if (parent) {
            var absPos = Core.getAbsolutePos(parent);
            Core.setPos(div, absPos.x, (absPos.y + parent.offsetHeight));
        } else {        
     		var size = Core.getWindowSize();
     		var h = (size.height - div.offsetHeight) / 2;
            Core.setPos(div, (size.width - div.offsetWidth) / 2,
                h > 0? h : 100);
        }

        this.showSelects(div, true);
        
        if (!inArray(div, this.dialogs)) {
            this.dialogs.push(div);
        }
    },
    hide : function(div) {
        this.showSelects(div, true);
        div = typeof div == "string"? $(div) : div;

        if (div) {
            div.style.display = "none";
        }
    },
    hide1 : function(div) {

		        this.showSelects(div, false);


        div = typeof div == "string"? $(div) : div;

        if (div) {
            div.style.display = "none";
        }
    },

    showSelects : function(div, value) {
        if (Core.isIE) {
		    var selects = document.getElementsByTagName("select");
		    var pos = Core.getAbsolutePos(div);
		    var w = div.offsetWidth;
		    var h = div.offsetHeight;
		    
		    for (var i = 0; i < selects.length; i++) {
		    	var s = selects[i];
		    	var p = s.parentNode;
		    	
		    	while (p && p != div) {
		    	    p = p.parentNode;
		    	}

                if (p != div) {
                    var ps = Core.getAbsolutePos(s);
          	        var ws = s.offsetWidth;
                    var hs = s.offsetHeight;

                    if (ps.x + ws > pos.x && ps.x < pos.x + w
                        && ps.y + hs > pos.y && ps.y < pos.y + h) {
    	                s.style.visibility = value? "" : "hidden";
    	            }
			    }
		    }
        }
    }
};var Event = {
    cache :null,
    add_ : function() {
        if (!Event.cache) {
            Event.cache = [];
            Event.attach(window, "unload", Event.flush_);
        }

        Event.cache.push(arguments);
    },
    flush_ : function() {
        for (var i = Event.cache.length - 1; i >= 0; i--) {
            var item = Event.cache[i];
            Event.detach(item[0], item[1], item[2], item[3]);
        }
        
        if (Event.onunload) {
            Event.onunload();
        }
        
        Event.onunload = null;
    },
    onunload : null
};

if (Core.isIE) {
    Event.attach = function(target, name, handler, phase) {
    	var func = function(evt) { handler.call(window.event.srcElement, evt); };
        target.attachEvent("on" + name, func);
        this.add_(target, name, func, phase);
    };

    Event.detach = function(target, name, handler, phase) {
        target.detachEvent("on" + name, handler);
    };

    Event.getTarget = function() {
        return window.event.srcElement;
    };
    
    Event.dispatch = function(target, name) {
        target.fireEvent("on" + name, document.createEventObject())
    };
} else {
    Event.attach = function(target, name, handler, phase) {
        if (target == window && !window.addEventListener) {
            target = document;
        }

        target.addEventListener(name, handler, phase);
        this.add_(target, name, handler, phase);
    };
    
    Event.detach = function(target, name, handler, phase) {
        if (target == window && !window.addEventListener) {
            target = document;
        }

        target.removeEventListener(name, handler, phase);
    };

    Event.getTarget = function(ev) {
        return ev.target;
    };
    
    Event.dispatch = function(target, name) {
        var event = document.createEvent("Events");
        event.initEvent(name, true, true);
        target.dispatchEvent(event);
    };
}

var I8N = {
    messages : null,
    lang : null,
    langs : [ "es", "gl", "nn-NO", "nb-NO"],
    get : function(key) {
        if (I8N.messages == null) {
            var lan = navigator.language || navigator.userLanguage;
            var finded = inArray(lan, I8N.langs);

            if (!finded) {
	            var ind = lan.indexOf("-");
	            
	            if (ind != -1) {
	                lan = lan.substring(0, ind);
	            }
	            
	            finded = inArray(lan, I8N.langs);
            }

            var filename = "messages" + (finded? "_" + lan.replace("-", "_") : "");
            I8N.messages = Core.loadProperties(filename);
        }

        return I8N.messages[key];
    },
    parse : function(str, pattern) {
        if (str == null || str.match("^\\s*$")) {
            return null;
        }

        if (!pattern) pattern = I8N.get("format.datetime");
        var d = new Date();
        I8N._parse(d, "Year", str, pattern, "yyyy");
        I8N._parse(d, "Month", str, pattern, "MM");
        I8N._parse(d, "Date", str, pattern, "dd");
        I8N._parse(d, "Hours", str, pattern, "hh");
        I8N._parse(d, "Minutes", str, pattern, "mm");
        I8N._parse(d, "Seconds", str, pattern, "ss");

        return d;
    },
    format : function(date, pattern) {
        if (!date) {
            return "";
        }

        if (!pattern) pattern = I8N.get("format.datetime");

        var str = I8N._format(pattern, date.getDate(), "dd");
        str = I8N._format(str, date.getMonth() + 1, "MM");
        str = I8N._format(str, date.getYear() < 1000? 1900 + date.getYear() : date.getYear(), "yyyy");
        str = I8N._format(str, date.getSeconds(), "ss");
        str = I8N._format(str, date.getMinutes(), "mm");
        str = I8N._format(str, date.getHours(), "hh");

        return str;
    },
    parseDate : function(str) {
        return I8N.parse(str, I8N.get("format.date"));
    },
    formatDate : function(str) {
        return I8N.format(str, I8N.get("format.date"));
    },
    formatNumber : function(number, decimals) {
    	if (isNaN(number)) return number;

    	var value = "" + number;
		var index = value.indexOf(".");
		var integer = parseInt(index != -1? value.substring(0, index) : value);
		var decimal = index != -1? value.substring(index + 1) : "";
		var signo = I8N.get("format.decimal");

    	return integer
    		+ (decimals > 0? signo + zeros(decimal, decimals, true) 
    		: (decimal? signo + decimal : ""));
    },
    parseNumber : function(value) {
		var signo = I8N.get("format.decimal");

		if(!value.match("^[-+]?[0-9]*[\\" + signo + "[0-9]*]?$")) {
			throw "Invalid number " + value;
		}

		var index = value.indexOf(signo);
		var integer = parseInt(index != -1? value.substring(0, index) : value);
		var decimal = index != -1? value.substring(index + 1) : null;
		
		return integer + (decimal? "." + decimal : "");
    },
    _format : function(returnValue, value, el) {
        return returnValue.replace(el, zeros(value, el.length));
    },
    _parse : function(date, prop, str, format, el) {
        var index = format.indexOf(el);
        
        if (index != -1) {
            var val = str.substr(index, el.length);
            
            if (val.charAt(0) == '0') val = val.substring(1);
            
            val = parseInt(val);
        
            if (isNaN(val)) {
                throw "Error parsing date " + str + " with pattern " + format;
            }

            date["set" + prop](prop == "Month"? val - 1 : val);
        }
    }
};
var NumberList = function(parent, className, input, min, max, minlengh) {
    this.element = Core.createElement("ul", parent, null, className);
    this.move = 0;
    this.input = input;
    this.min = min;
    this.max = max;
    this.minlength = minlengh || 1;
    var list = this;

    this.createChild("+", function() { list.start(1); }, function() { list.stop(); } );

    for (var i = 0; i < 7; i++) {
        this.createChild(" ", function(event) {
            list.input.value = Event.getTarget(event).childNodes[0].nodeValue;
            list.close();
            Event.dispatch(list.input, "change");
        } );
    }
    
    this.createChild("-", function() { list.start(-1); }, function() { list.stop(); } );
};

NumberList.prototype.show = function() {
	var input = this.input;
    this.current = parseInt(input.value);
    this.refresh();
    Dialog.show(this.element, input);
};

NumberList.prototype.close = function() {
    Dialog.hide1(this.element);
}; 

NumberList.prototype.createChild = function(content, handler, handler2) {
    var child = Core.createElement("li", this.element, content);
    Core.initHover(child);

    if (handler2) {
        Event.attach(child, "mousedown", handler);
        Event.attach(child, "mouseup", handler2);
    } else {
        Event.attach(child, "click", handler);
    }
};
    
NumberList.prototype.refresh = function()  {
    var childs = this.element.childNodes;
    var cur = this.current;
    
    if (cur >= this.max - 3) {
        cur = this.max - 3;
    } else if (cur <= this.min + 3) {
    	cur = this.min + 3;
    }
    
    var top = cur + 4;

    for (var i = 1; i < 8; i++) {
        Core.setClass(childs[i], "hover", false);
        var str = (top - i) + "";
        
        for (; str.length < this.minlength; str = '0' + str);
        
        childs[i].firstChild.nodeValue = str;
    }
};

NumberList.prototype.start = function(value) {
    this.move = value;
    NumberList.current = this;
    this.run();
};
    
NumberList.prototype.stop = function() {
    this.move = 0;
};

NumberList.prototype.run = function() {
    if (   (this.move > 0 && this.current + 3 < this.max)
        || (this.move < 0 && this.current - 3> this.min)) {
        this.current += this.move;
        this.refresh();
        var list = this;
        setTimeout("NumberList.current.run()", 60);
    }
};

NumberList.current = null;
function $(id) {
    return document.getElementById(id);
};

function forEach(object, block) {
    var args = [];
   
    for (var i = 0; i < arguments.length - 2; i++) {
        args[i] = arguments[i + 2];
    }

    if (object) {
        if (typeof object.length == "number") {
            for (var i = 0; i < object.length; i++) {
            	var obj = object[i];
            	var func = typeof block == "string"? obj[block] : block;
                func.apply(obj, args);
            }
        } else {
            for (var key in object) {
            	var obj = object[key];
            	var func = typeof block == "string"? obj[block] : block;
                func.apply(obj, args);
            }   
        }
    }
};

function assert(condition, message) {
    if (!condition && DebugConsole.isOpen()) {
        DebugConsole.write("Assertion failed: " + message);
        var callstack = null;

        if (arguments.caller) { // Internet Explorer
            this.callstack = [];
    
            for (var caller = arguments.caller; caller != null; caller = caller.caller) {
                this.callstack.push(caller.name ? caller.name : "<anonymous>");
            }
        } else {
            try {
                var x; x.y;
            } catch (exception) {
                this.callstack = exception.stack.split("\n");
            }
        }

        if (this.callstack) {
            for (var i in this.callstack) {
                DebugConsole.write("> " + this.callstack[i]);
            }
        }

        throw message;
    }
};

function inArray(value, array) {
    for (var i = 0; i < array.length; i++) {
        if (value == array[i]) {
            return true;
        }
    }
    
    return false;
};

function zeros(value, length, right) {
	var res = "" + value;

	for (; res.length < length; res = right? res + '0' : '0' + res);

	return res;
};

function Schema(ns, name, prefixes) {
	assert(ns && !Schema.all[ns], "Needed schema name or exists one schema with that namespace");
	this.name = name;
	this.ns = ns;
	this.types = {};
	this.prefixes = prefixes || {};
	Schema.all[ns] = this;
};

Schema.all = {};

Schema.prototype.getType = function(name) {
	if (name.indexOf(":") != -1) {
		var res = name.split(":");
		var prefix = res[0];
		var ns = this.prefixes[prefix];

		if (ns) {
			return Schema.getTypeNS(ns, res[1]);
		}

		return Schema.getType(name);
	}

	var type = this.types[name];

	if (!type) {
		alert("Type " + name + " not defined");
		throw "Error";
	}

	return type;
};

Schema.getType = function(name) {
	var res = name.split(":");
	return Schema.getTypeNS(Schema.prefixes[res[0]], res[1]);
};

Schema.getTypeNS = function(ns, name) {
	var schema = Schema.all[ns];
	
	if (!schema) {
		alert("Schema " + ns + " not defined");
		throw "Error";
	}
	
	var type = schema.types[name];	

	if (!type) {
		alert("Type " + name + " not defined");
		throw "Error";
	}

	return type;
};

Schema.get = function(ns) {
	var schema = Schema.all[ns];

	if (!schema) {
		schema = new Schema(ns);
	}
	
	return schema;
};

Schema.prefixes = {
	"xsd_" : "http://www.w3.org/2001/XMLSchema"
};

Schema.registerPrefix = function(prefix, namespace) {
	this.prefixes[prefix] = namespace;
};
function Type() {
};

Type.prototype.setSchema = function(schema) {
	this.schema = schema;
	return this;
};

Type.prototype.setName = function(name) {
	this.name = name;
	this.schema.types[name] = this;
	return this;
};

Type.prototype.canonicalValue = function(value) {
	value = value.toString();

	switch (this.whiteSpace) {
		case "replace": value = value.replace(/[\t\r\n]/g, " "); break;
		case "collapse": value = value.replace(/[\t\r\n ]+/g, " ").replace(/^\s+|\s+$/g, ""); break;
	}

	return value;
};


Type.prototype.getMaxLength = function() {
	return this.maxLength != null? this.maxLength 
		: (this.length != null? this.length
			: (this.totalDigits != null? this.totalDigits + 1 : null));
};

Type.prototype.getDisplayLength = function() {
	return this.displayLength != null? this.displayLength 
		: this.getMaxLength();
};function AtomicType() {
	this.patterns = [];
};

AtomicType.prototype = new Type;

AtomicType.prototype.setBase = function(base) {
	var baseType = typeof base == "string"? this.schema.getType(base) : base;

	for (var id in baseType)  {
		var value = baseType[id];

		if (id == "patterns") {
			copyArray(value, this.patterns);
		} else if (id != "name") {
			this[id] = value;
		}
	}
	
	return this;
};

AtomicType.prototype.put = function(name, value) {
	if (name == "base") {
		this.setBase(value);
	} else if (name == "pattern") {
		copyArray([value], this.patterns);
	} else {
		this[name] = value;
	}
	
	return this;
};

/** If valid return canonicalValue else null*/
AtomicType.prototype.validate = function (value) {
	value = this.canonicalValue(value);

	for (var i = 0; i < this.patterns.length; i++) {
		if (!value.match(this.patterns[i])) {
			return false;
		}
	}

	if (this.enumeration != null) {
		var matched = false;

		for (var i = 0; i < this.enumeration.length; i++) {
			if (value == this.canonicalValue(this.enumeration[i])) {
				matched = true;
				break;
			}
		}

		if (!matched) {
			return false;
		}
	}

	var l = value.length;

	if (   (this.length != null && this.length != l)
		|| (this.minLength != null && l < this.minLength)
		|| (this.maxLength != null && l > this.maxLength)
		|| (this.maxInclusive != null && value > this.maxInclusive)
		|| (this.maxExclusive != null && value >= this.maxExclusive)
		|| (this.minInclusive != null && value < this.minInclusive)
		|| (this.minExclusive != null && value <= this.minExclusive) ) {
		return false;
	}
	
	if (this.totalDigits != null || this.fractionDigits != null) {
		var index = value.indexOf(".");
		var integer = parseInt(index != -1? value.substring(0, index) : value);
		var decimal = index != -1? value.substring(index + 1) : "";
		
		if (index != -1) {
			var i = decimal.length - 1;
			for (; i >= 0 && decimal.charAt(i) == 0; i--);
			decimal = decimal.substring(0, i + 1);
		}

		if (   (this.totalDigits != null && integer.length + decimal.length > this.totalDigits)
			|| (this.fractionDigits != null && decimal.length > this.fractionDigits)) {
			return false;
		}
	}
	
	return true;
};

AtomicType.prototype.normalize = function (value) {
	if (this.fractionDigits != null) {
		var number = parseFloat(value);
		var num;

		if (isNaN(number)) {
			return "NaN";
		}

		if (number == 0) {
			num = zeros(0, this.fractionDigits + 1, true);
		}  else {
			var mult = zeros(1, this.fractionDigits + 1, true);
			num = "" + Math.round(number * mult);
		}

		if (this.fractionDigits != 0) {
			var index = num.length - this.fractionDigits;
			return (index == 0? "0" : num.substring(0, index)) + "." + num.substring(index);
		}
		
		return num;
	}
	
	return value;
};
var TypeDefs = {
	initAll : function() {
		this.init("http://www.w3.org/2001/XMLSchema", this.Default);
		this.init("http://www.w3.org/2002/xforms", this.XForms);
	},
	init : function(ns, list) {
		var schema = Schema.get(ns);
	
		for (var id in list) {
			var type = new AtomicType();
			type.setSchema(schema);
			type.setName(id);
			var props = list[id];
			var base = props["base"];

			if (base) {
				type.setBase(base);
			}
		
			for (var prop in props) {
				if (prop != "base") {				
					type[prop] = props[prop];
				}
			}
		}
	},
	ctes : {
		i : "A-Za-z_\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\xFF",
		c : "A-Za-z_\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\xFF\\-\\.0-9\\xB7"
	}
};

TypeDefs.Default = {
	"string" : {
		"whiteSpace" : "preserve"
	},
	"boolean" : {
		"patterns" : [ "^true|false|0|1$" ],
		"class" : "boolean"
	},
	"decimal" : {
		"patterns" : [ "^[-+]?[0-9]*[\\.[0-9]*]?$" ],
		"class" : "number",
		"displayLength" : 8,
		"format" : function(value) {
			return I8N.formatNumber(value, this.fractionDigits);
		},
		"parse" : function(value) {
			return I8N.parseNumber(value);
		}
	},
	"float" : {
		"patterns" : [ "^(([-+]?([0-9]+(\\.[0-9]*)?)|(\\.[0-9]+))([eE][-+]?[0-9]+)?|-?INF|NaN)$" ],
		"class" : "number"
	},
	"double" : {
		"patterns" : [ "^(([-+]?([0-9]+(\\.[0-9]*)?)|(\\.[0-9]+))([eE][-+]?[0-9]+)?|-?INF|NaN)$" ],
		"class" : "number"
	},
	"datetime" : {
		"patterns" : [ "^([12][0-9]{3})-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])T([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9](\.[0-9]+)?(Z|[+-]([01][0-9]|2[0-3]):[0-5][0-9])?$" ],
		"class" : "datetime",
		"displayLength" : 20,
		"format" : function(value) {
			return I8N.format(I8N.parse(value, "yyyy-MM-ddThh:mm:ss"));
		},
		"parse" : function(value) {
			return I8N.format(I8N.parse(value), "yyyy-MM-ddThh:mm:ss");
		}
	},
	"date" : {
		"patterns" : [ "^([12][0-9]{3})-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])(Z|[+-]([01][0-9]|2[0-3]):[0-5][0-9])?$" ],
		"class" : "date",
		"displayLength" : 10,
		"format" : function(value) {
			return I8N.formatDate(I8N.parse(value, "yyyy-MM-dd"));
		},
		"parse" : function(value) {
			return I8N.format(I8N.parseDate(value), "yyyy-MM-dd");
		}
	},
	"time" : {
		"patterns" : [ "^([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9](\.[0-9]+)?(Z|[+-]([01][0-9]|2[0-3]):[0-5][0-9])?$" ],
		"displayLength" : 8
	},
	"duration" : {
		"patterns" : [ "^P([0-9]+Y)?([0-9]+M)?([0-9]+D)?(T([0-9]+H)?([0-9]+M)?([0-9]+S)?)?$" ]
	},
	"gDay" : {
		"patterns" : [ "^(0[1-9]|[12][0-9]|3[01])$" ]
	},
	"gMonth" : {
		"patterns" : [ "^(0[1-9]|1[012])$" ]
	},
	"gMonthDay" : {
		"patterns" : [ "^(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])$" ]
	},
	"gYear" : {
		"patterns" : [ "^([12][0-9]{3})$" ]
	},
	"gYearMonth" : {
		"patterns" : [ "^([12][0-9]{3})-(0[1-9]|1[012])$" ]
	},
	"integer" : {
		"base" : "xsd_:decimal",
		"fractionDigits" : "0"
	},
	"nonPositiveInteger" : {
		"base" : "xsd_:integer",
		"patterns" : [ "^[-][0-9]*$" ]
	},
	"nonNegativeInteger" : {
		"base" : "xsd_:integer",
		"patterns" : [ "^[+][0-9]*$" ]
	},
	"negativeInteger" : {
		"base" : "xsd_:integer",
		"patterns" : [ "^[-][0-9]*$" ]
	},
	"positiveInteger" : {
		"base" : "xsd_:integer",
		"patterns" : [ "^[+][0-9]*$" ]
	},
	"normalizedString" : {
		"whiteSpace" : "replace"
	},
	"token" : {
		"whiteSpace" : "collapse"
	},
	"language" : {
		"base" : "xsd_:token",
		"patterns" : [ "^[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*$" ]
	},
	"name" : {
		"base" : "xsd_:token",
		"patterns" : [ "^[" + TypeDefs.ctes.i + ":][" + TypeDefs.ctes.c + ":]*$" ]
	},
	"NCName" : {
		"base" : "xsd_:token",
		"patterns" : [ "^[" + TypeDefs.ctes.i + "][" + TypeDefs.ctes.c + "]*$" ]
	},
	"ID" : {
		"base" : "xsd_:NCName"
	},
	"IDREF" : {
		"base" : "xsd_:NCName"
	}
};

TypeDefs.XForms = {
	"email" : {
		"base" : "xsd_:string",
		"whiteSpace" : "collapse",
		"patterns" : [ "^[A-Za-z0-9\*\+\-]+(\.[A-Za-z0-9\*\+\-]+)*@[A-Za-z0-9\*\+\-]+(\.[A-Za-z0-9\*\+\-]+)*$" ]
	},
	"ID-card-number" : {
		"base" : "xsd_:string",
		"minLength" : 12,
		"maxLength" : 19,
		"patterns" : [ "^[0-9]+$" ]
	},
	"url" : {
		"base" : "xsd_:string",
		"whiteSpace" : "collapse",
		"pattern" : [ "^(ht|ft)tp(s?)://([a-z0-9]*:[a-z0-9]*@)?([a-z0-9.]*\.[a-z]{2,7})$" ]
	},
	"amount" : {
		"base" : "xsd_:decimal",
		"format" : function(value) {
			return I8N.formatNumber(value, 2);
		}
	}
};

TypeDefs.initAll();function ListType() {
	this.whiteSpace = "collapse";
};

ListType.prototype = new Type;

ListType.prototype.setItemType = function(itemType) {
	this.itemType = typeof itemType == "string"? this.schema.getType(itemType) : itemType;
	return this;
};

ListType.prototype.validate = function (value) {
	var items = this.baseType.canonicalValue.call(this, value).split(" ");
	value = "";

	if (items.length == 1 && items[0] == "") {
		items = [];
    }

	for (var i = 0; i < items.length; i++) {
		var item = itemType.validate(items[i]);

		if (!item) {
			return null;
		}
		
		value += value.length == 0? item : " " + item;
	}

	if (   (this.length != null > 0 && this.length != l)
		|| (this.minLength != null && l < this.minLength)
		|| (this.maxLength != null && l > this.maxLength)) {
		return null;
	}

    return null;
};

ListType.prototype.canonicalValue = function(value) {
	var items = this.baseType.canonicalValue(value).split(" ");
	var value = "";

	for (var i = 0; i < items.length; i++) {
		var item = this.itemType.canonicalValue(items[i]);
		value += (value.length == 0? "" : " ") + item;
    }

    return value;
};



function UnionType() {
	this.baseTypes = [];
};

UnionType.prototype = new Type;

UnionType.prototype.addType = function(type) {
	this.baseTypes.push(typeof type == "string"? this.schema.getType(type) : type);
	return this;
};

UnionType.prototype.validate = function (value) {
	for (var i = 0; i < this.baseTypes.length; ++i) {
		value = this.baseTypes[i].validate(value);
		
		if (value) {
			return null;
		}
	}

	return value;
};function getValue(node, format) {
	assert(node);
	var value = node.nodeType == NodeType.ATTRIBUTE? node.nodeValue :
		(node.firstChild != null? node.firstChild.nodeValue : "");

	if (value && format && node.type.format) {
		try { value = node.type.format(value); } catch(e) { }
	}

	return value;
};

function setValue(node, value) {
	assert(node);

	if (node.nodeType == NodeType.ATTRIBUTE) {
		node.nodeValue = value;
	} else if (node.firstChild) {
		node.firstChild.nodeValue = value;
	} else {
		node.appendChild(node.ownerDocument.createTextNode(value));
	}
};

function run(action, element, evt, synch) {
	xforms.openAction();
    
	if (synch) {
		Dialog.show("statusPanel");

		setTimeout(function() { 
			action.execute(IdManager.find(element), null, evt);
			Dialog.hide("statusPanel");
			xforms.closeAction();
		}, 1 );
	} else {
		action.execute(IdManager.find(element), null, evt);
		xforms.closeAction();
	}
};

function show(el, type, value) {
	el.parentNode.lastChild.style.display = value? 'inline' : 'none';
};

String.prototype.trim = function() {
	return this.replace(/^\s+|\s+$/, '');
};

function copyArray(source, dest) {
	for (var i = 0; i < source.length; i++) {
		dest[i] = source[i];
	}
};
