<?xml version="1.0" encoding="UTF-8"?>
<!--
   ===================================================================================================================
   XSL:     aris2xpdl.xsl

   DESCRIPTION:
   This xslt will convert ARIS XML to XPDL1.0

   ===================================================================================================================
-->

<!-- Define constants -->
<!DOCTYPE xsl:stylesheet [
   <!ENTITY crlf              '&#xA;'>
   <!ENTITY arisParticROLE    'OT_POS'>
   <!ENTITY particROLE        'ROLE'>
   <!ENTITY arisParticHUMAN   'OT_PERS OT_PERS_TYPE OT_EMPL_INST'>
   <!ENTITY particHUMAN       'HUMAN'>
   <!ENTITY arisParticOU      'OT_ORG_UNIT OT_ORG_UNIT_TYPE OT_GRP'>
   <!ENTITY particOU          'ORGANIZATIONAL_UNIT'>
   <!ENTITY arisParticSYSTEM  'OT_SYS_ORG_UNIT OT_SYS_ORG_UNIT_TYPE OT_APPL_SYS OT_APPL_SYS_CLS OT_APPL_SYS_TYPE OT_MOD_TYPE OT_MOD'>
   <!ENTITY particSYSTEM      'SYSTEM'>
   <!ENTITY arisParticipants  '&arisParticROLE; &arisParticHUMAN; &arisParticOU; &arisParticSYSTEM;'>
   <!ENTITY arisDataFields    'OT_ERM_ATTR OT_TECH_TRM OT_ATTR_TYPE_GRP'>
   <!ENTITY arisFunctions     'OT_FUNC'>
   <!ENTITY arisRules         'OT_RULE'>
   <!ENTITY arisEvents        'OT_EVT'>
   <!ENTITY arisKnownObjs     '&arisParticipants; &arisDataFields; &arisFunctions; &arisRules; &arisEvents;'>
   <!ENTITY true              'TRUE'>
   <!ENTITY false             'FALSE'>
]>

<xsl:stylesheet version="1.0"
   xmlns:xpdl="http://www.wfmc.org/2002/XPDL1.0"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   exclude-result-prefixes="xpdl">

   <xsl:output method="xml" version="1.0" encoding="UTF-8"
      indent="yes" />

   <!--
      ===================================================================================================================
      XSLT processing start point (process whole document)
      ===================================================================================================================
   -->
   <xsl:template match="/">

      <xsl:apply-templates />

   </xsl:template>

   <!--
      ===================================================================================================================
      AML = head element of the ARIS XML document.  This will add the xpdl:Package element
      ===================================================================================================================
   -->
    <xsl:template match="aml|Aml|AML">
      <xpdl:Package Id="newpkg"
             Name="newpkg"
             xmlns="http://www.wfmc.org/2002/XPDL1.0"
             xmlns:xpdl="http://www.wfmc.org/2002/XPDL1.0"
             xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
             xsi:schemaLocation="http://www.wfmc.org/2002/XPDL1.0 http://wfmc.org/standards/docs/TC-1025_schema_10_xpdl.xsd">

         <xsl:attribute name="Id">
            <xsl:for-each select="Group/Model/AttrDef[@AttrDef.Type='AT_ID']">
               <xsl:value-of select="AttrValue"/>
            </xsl:for-each>
         </xsl:attribute>
         <xsl:attribute name="Name">
            <xsl:for-each select="Group/Model/AttrDef[@AttrDef.Type='AT_NAME']">
               <xsl:value-of select="AttrValue"/>
            </xsl:for-each>
         </xsl:attribute>

         <!-- Output the package header, redefinable header and script -->
         <xsl:apply-templates select="Header-Info" />
         <!-- Output main section -->
         <xsl:call-template name="outputMain">
            <xsl:with-param name="groups" select="//Group" />
         </xsl:call-template>

         <xpdl:ExtendedAttributes>
            <xpdl:ExtendedAttribute Name="ALLOW_UNDEFINED_VARIABLES" Value="true"/>
            <xpdl:ExtendedAttribute Name="DYNAMIC_VARIABLE_HANDLING" Value="true"/>
         </xpdl:ExtendedAttributes>
         
      </xpdl:Package>

   </xsl:template>

   <!--
      ===================================================================================================================
      Header-Info = this will add the xpdl:PackageHeader, xpdl:RedefinableHeader and xpdl:Script
      ===================================================================================================================
   -->
   <xsl:template match="Header-Info">
       <xpdl:PackageHeader>
           <xpdl:XPDLVersion>1.0</xpdl:XPDLVersion>
         <xpdl:Vendor>Together</xpdl:Vendor>
         <xpdl:Created>
            <xsl:value-of select="@CreateDate"/><xsl:value-of select="' '"/><xsl:value-of select="@CreateTime"/>
         </xpdl:Created>
       </xpdl:PackageHeader>
       <xpdl:RedefinableHeader>
         <xpdl:Author>
            <xsl:value-of select="AttrDef[@AttrDef.Type = 'AT_CREATOR']/AttrValue" />
         </xpdl:Author>
         <xpdl:Version>1.0</xpdl:Version>
      </xpdl:RedefinableHeader>

      <!-- TODO - read from aris witch scripting language is set -->
      <xpdl:Script Type="text/javascript"/>

   </xsl:template>
   <!--
      ===================================================================================================================
      outputMain = This will output the main section of the xpdl including participants, datafields, artifacts and the processes
      ===================================================================================================================
   -->
   <xsl:template name="outputMain">
      <xsl:param name="groups" />

      <xsl:variable name="objDef" select="$groups/ObjDef" />
      <xsl:variable name="textDef" select="$groups/FFTextDef" />
      <xsl:variable name="models" select="$groups/Model[starts-with (@Model.Type, 'MT_EEPC')]" />

      <!-- Add any participants -->
      <xpdl:Participants>
         <xsl:apply-templates select="$objDef" mode="participants" />
      </xpdl:Participants>

      <!-- Add any data fields -->
      <xpdl:DataFields>
         <xsl:apply-templates select="$objDef" mode="dataFields" />
      </xpdl:DataFields>

      <!-- Add each process -->
      <xpdl:WorkflowProcesses>
         <xsl:for-each select="$models">

            <!-- Add processes for each EPC model -->
               <xsl:apply-templates select=".">
               <xsl:with-param name="objDefinitions" select="$objDef" />
               </xsl:apply-templates>
         </xsl:for-each>
      </xpdl:WorkflowProcesses>

   </xsl:template>

   <!--
      ===================================================================================================================
      ObjDef (participants) - Identify participants and add them to the xpdl
      ===================================================================================================================
   -->
   <xsl:template match="ObjDef" mode="participants">

      <!-- Get ID -->
      <xsl:variable name="objDefID" select="@ObjDef.ID" />
      <!-- Only interested in participant type objects -->
      <xsl:if test="contains('&arisParticipants;', @TypeNum)">
         <!-- Only interested with objects that have an occurrence within one of the models -->
         <xsl:if
            test="count (..//Model/ObjOcc[@ObjDef.IdRef = $objDefID]) &gt; 0">
            <!-- Identify participants -->
            <xsl:variable name="type">
               <xsl:choose>
                  <xsl:when
                     test="contains('&arisParticROLE;', @TypeNum)">
                     <xsl:text>&particROLE;</xsl:text>
                  </xsl:when>
                  <xsl:when
                     test="contains('&arisParticHUMAN;', @TypeNum)">
                     <xsl:text>&particHUMAN;</xsl:text>
                  </xsl:when>
                  <xsl:when
                     test="contains('&arisParticOU;', @TypeNum)">
                     <xsl:text>&particOU;</xsl:text>
                  </xsl:when>
                  <xsl:when
                     test="contains('&arisParticSYSTEM;', @TypeNum)">
                     <xsl:text>&particSYSTEM;</xsl:text>
                  </xsl:when>
               </xsl:choose>
            </xsl:variable>

            <!-- If participant then add it -->
            <xsl:if test="$type != ''">
               <xpdl:Participant Id="{AttrDef[@AttrDef.Type='AT_ID']/AttrValue}"
                  Name="{normalize-space(AttrDef[@AttrDef.Type = 'AT_NAME']/AttrValue)}">
                  <xpdl:ParticipantType Type="{$type}" />
                  <!-- If there is a description then add it -->
                  <xsl:if test="AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue">
                     <xpdl:Description>
                           <xsl:value-of select="AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue"/>
                     </xpdl:Description>
                  </xsl:if>
               </xpdl:Participant>
            </xsl:if>

         </xsl:if>
      </xsl:if>

   </xsl:template>

   <!--
      ===================================================================================================================
      ObjDef (dataFields) - Identify data fields and add them to the xpdl
      ===================================================================================================================
   -->
   <xsl:template match="ObjDef" mode="dataFields">

      <!-- Get object ID -->
      <xsl:variable name="objDefID" select="@ObjDef.ID" />
      <!--  Only interested in data types -->
      <xsl:if test="contains ('&arisDataFields;', @TypeNum)">
         <!-- Only interested with objects that have an occurrence within one of the models -->
         <xsl:if
            test="count (..//Model/ObjOcc[@ObjDef.IdRef = $objDefID]) &gt; 0">
            <xpdl:DataField Name="{normalize-space (AttrDef[@AttrDef.Type = 'AT_NAME']/AttrValue)}">
			   <xsl:attribute name="Id">
				<xsl:value-of select="AttrDef[@AttrDef.Type='AT_ID']/AttrValue"/>
			   </xsl:attribute>
               <xpdl:DataType>
                  <xpdl:BasicType Type="STRING" />
               </xpdl:DataType>
               <!-- If there is a description then add it -->
               <xsl:if test="AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue">
                  <xpdl:Description>
                        <xsl:value-of select="normalize-space(AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue)"/>
                  </xpdl:Description>
               </xsl:if>
            </xpdl:DataField>
         </xsl:if>
      </xsl:if>

   </xsl:template>

   <!--
      ===================================================================================================================
      Model = Add WorkflowProcess for each model in this group
      ===================================================================================================================
   -->
   <xsl:template match="Model">
      <xsl:param name="objDefinitions"/>

      <xpdl:WorkflowProcess Id="{AttrDef[@AttrDef.Type = 'AT_ID']/AttrValue}"  Name="{normalize-space(AttrDef[@AttrDef.Type = 'AT_NAME']/AttrValue)}">
         <xpdl:ProcessHeader>
            <xpdl:Created><xsl:value-of select="AttrDef[@AttrDef.Type = 'AT_CREAT_TIME_STMP']/AttrValue" /></xpdl:Created>
         </xpdl:ProcessHeader>
         <xpdl:RedefinableHeader />

         <!-- Add the activities -->
         <xpdl:Activities>
            <xsl:apply-templates select="ObjOcc">
               <xsl:with-param name="objDefinitions" select="$objDefinitions" />
            </xsl:apply-templates>
         </xpdl:Activities>

         <!-- Add the transitions -->
         <xpdl:Transitions>
            <xsl:apply-templates  select="ObjOcc/CxnOcc">
               <xsl:with-param name="objDefinitions" select="$objDefinitions" />
            </xsl:apply-templates>
         </xpdl:Transitions>
      </xpdl:WorkflowProcess>
   </xsl:template>

   <!--
===================================================================================================================
   ObjOcc = this will add the Activity entries
===================================================================================================================
-->
   <xsl:template match="ObjOcc">
      <xsl:param name="objDefinitions" />

      <xsl:variable name="objOccId" select="@ObjOcc.ID" />

      <!-- Get the object definition of this occurrence -->
      <xsl:variable name="objDefId" select="@ObjDef.IdRef" />
      <xsl:variable name="objDef" select="$objDefinitions[@ObjDef.ID = $objDefId]" />

      <xsl:variable name="objID" select="$objDef/AttrDef[@AttrDef.Type='AT_ID']/AttrValue" />

      <!-- Check if this object will be drawn as an activity -->
      <xsl:variable name="isActivity">
         <xsl:call-template name="isActivity">
            <xsl:with-param name="objOcc" select="." />
            <xsl:with-param name="objDef" select="$objDef" />
            <xsl:with-param name="allObjOccs" select="../ObjOcc" />
         </xsl:call-template>
      </xsl:variable>

      <!-- If this is an activity then draw it -->
      <xsl:if test="$isActivity = '&true;'">

         <xpdl:Activity Id="{$objID}" Name="{normalize-space ($objDef/AttrDef[@AttrDef.Type = 'AT_NAME']/AttrValue)}">
            <!-- If there is a description then add it -->
            <xsl:if test="$objDef/AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue">
               <xpdl:Description>
                  <xsl:value-of select="normalize-space($objDef/AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue)"/>
               </xpdl:Description>
            </xsl:if>

            <!-- Check for type of activity -->
            <xsl:choose>
               <!-- Event -->
               <xsl:when test="contains('&arisEvents;', $objDef/@TypeNum)">
                  <xpdl:Route/>
               </xsl:when>

               <!-- Route -->
               <xsl:when test="contains('&arisRules;', $objDef/@TypeNum)">
                  <!-- Determine type of route -->
                  <xsl:variable name="type">
                     <xsl:choose>
                        <xsl:when test="starts-with($objDef/@SymbolNum, 'ST_OPR_OR') or starts-with($objDef/@SymbolNum, 'ST_OR')">
                           <xsl:message terminate="yes"> OR object is not supported in XPDL1.0!</xsl:message>
                        </xsl:when>
                        <xsl:when test="starts-with($objDef/@SymbolNum, 'ST_OPR_AND') or starts-with($objDef/@SymbolNum, 'ST_AND')">
                           <xsl:text>AND</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                           <!-- Otherwise assume to be XOR route -->
                           <xsl:text>XOR</xsl:text>
                        </xsl:otherwise>
                     </xsl:choose>
                  </xsl:variable>

                  <xsl:variable name="hasSplit">
                     <xsl:choose>
                        <xsl:when test="count(CxnOcc) &gt; 1">
                           <xsl:text>true</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                           <xsl:text>false</xsl:text>
                        </xsl:otherwise>
                     </xsl:choose>
                  </xsl:variable>

                  <xsl:variable name="hasJoin">
                     <xsl:choose>
                        <xsl:when test="count(../ObjOcc/CxnOcc[@ToObjOcc.IdRef=$objOccId]) &gt; 1">
                           <xsl:text>true</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                           <xsl:text>false</xsl:text>
                        </xsl:otherwise>
                     </xsl:choose>
                  </xsl:variable>

                  <xpdl:Route/>

                  <xsl:if test="$hasJoin='true' or $hasSplit='true'">
                     <xpdl:TransitionRestrictions>
                        <xpdl:TransitionRestriction>
                           <xsl:if test="$hasJoin='true'">
                              <xpdl:Join Type="{$type}"/>
                           </xsl:if>
                           <xsl:if test="$hasSplit='true'">
                              <xpdl:Split Type="{$type}">
                                 <xpdl:TransitionRefs>
                                    <xsl:for-each select="../ObjOcc[@ObjOcc.ID=$objOccId]/CxnOcc">
                                    <xsl:variable name="cxnDefId" select="@CxnDef.IdRef" />
                                       <xpdl:TransitionRef>
                                          <xsl:attribute name="Id">
                                          <xsl:value-of select="$objDef/CxnDef[@CxnDef.ID = $cxnDefId]/AttrDef[@AttrDef.Type = 'AT_ID']/AttrValue"/>
                                          </xsl:attribute>
                                       </xpdl:TransitionRef>
                                    </xsl:for-each>
                                 </xpdl:TransitionRefs>
                              </xpdl:Split>
                           </xsl:if>
                        </xpdl:TransitionRestriction>
                     </xpdl:TransitionRestrictions>
                  </xsl:if>


               </xsl:when>

               <!-- Task -->
               <xsl:otherwise>
                  <xpdl:Implementation>
                     <!-- Check if this is a subflow call -->
                     <xsl:variable name="subFlow">
                        <xsl:if test="$objDef/@LinkedModels.IdRefs">
                           <xsl:call-template name="getFirstLinkedModel">
                              <xsl:with-param name="linkedRef" select="$objDef/@LinkedModels.IdRefs" />
                              <xsl:with-param name="models" select="../../Model" />
                           </xsl:call-template>
                        </xsl:if>
                     </xsl:variable>

                     <xsl:choose>
                        <!-- Subflow -->
                        <xsl:when test="$subFlow != ''">
                           <xpdl:SubFlow Id="{$subFlow}" />
                        </xsl:when>

                        <!-- Otherwise, this is only a task -->
                        <xsl:otherwise>
                           <xpdl:No/>
                        </xsl:otherwise>

                     </xsl:choose>

                  </xpdl:Implementation>
               </xsl:otherwise>

            </xsl:choose> <!-- type of activity -->

            <!-- Add any performers to the activity - only for task types -->
            <xsl:if test="contains ('&arisFunctions;', $objDef/@TypeNum)">
               <xsl:apply-templates select="." mode="performers">
                  <xsl:with-param name="objDefinitions" select="$objDefinitions" />
               </xsl:apply-templates>
            </xsl:if>

         </xpdl:Activity>
      </xsl:if>

   </xsl:template>

   <!--
===================================================================================================================
   ObjOcc (performers) = Identify any connected participants - then add as a performer.
===================================================================================================================
-->
   <xsl:template match="ObjOcc" mode="performers">
      <xsl:param name="objDefinitions" />

      <!-- Get this occurence ID -->
      <xsl:variable name="objOccID" select="@ObjOcc.ID" />

      <!-- Get all object occs connected to this object -->
      <xsl:variable name="connectedOccs" select="../ObjOcc[CxnOcc/@ToObjOcc.IdRef = $objOccID]" />

      <!-- Get object definitions of connected objects -->
      <xsl:variable name="connectedObjs" select="$objDefinitions[@ObjDef.ID = $connectedOccs/@ObjDef.IdRef]" />

      <!-- Get all participant objects from the connected list -->
      <xsl:variable name="participantObjs" select="$connectedObjs[contains('&arisParticipants;', @TypeNum)]" />

      <xsl:if test="count ($participantObjs) &gt; 0">
         <xpdl:Performer>
            <xsl:value-of select="$participantObjs/AttrDef[@AttrDef.Type='AT_ID']/AttrValue" />
         </xpdl:Performer>
      </xsl:if>

   </xsl:template>


   <!--
      ===================================================================================================================
      CxnOcc = Add the transitions
      ===================================================================================================================
   -->
   <xsl:template match="CxnOcc">
      <xsl:param name="objDefinitions" />

      <!-- Get this object occurrence -->
      <xsl:variable name="thisObjOcc" select="../." />
      <!-- Get the object definition of this occurrence -->
      <xsl:variable name="objDef" select="$objDefinitions[@ObjDef.ID = $thisObjOcc/@ObjDef.IdRef]" />
      <!-- Get all object occurrence of this model -->
      <xsl:variable name="objOccs" select="../../ObjOcc" />

      <!-- Check if this object will be drawn as an activity.  If not then don't draw it's transitions -->
      <xsl:variable name="isActivity">
         <xsl:call-template name="isActivity">
            <xsl:with-param name="objOcc" select="$thisObjOcc" />
            <xsl:with-param name="objDef" select="$objDef" />
            <xsl:with-param name="allObjOccs" select="$objOccs" />
         </xsl:call-template>
      </xsl:variable>

      <!-- If this is an activity then draw it's transitions -->
      <xsl:if test="$isActivity = '&true;'">

         <!-- Get the target object occurrence ID -->
         <xsl:variable name="toObjOccId" select="@ToObjOcc.IdRef" />
         <!-- Get the to object occurrence -->
         <xsl:variable name="toObjOcc" select="$objOccs[@ObjOcc.ID = $toObjOccId]" />
         <!-- Get the to object definition -->
         <xsl:variable name="toObjDef" select="$objDefinitions[@ObjDef.ID = $toObjOcc/@ObjDef.IdRef]" />
         <xsl:variable name="toObjID" select="$toObjDef/AttrDef[@AttrDef.Type='AT_ID']/AttrValue" />

         <!-- Get the source object occurrence ID -->
         <xsl:variable name="fromObjOccId" select="$thisObjOcc/@ObjOcc.ID" />
         <!-- Get the to object occurrence -->
         <xsl:variable name="fromObjOcc" select="$objOccs[@ObjOcc.ID = $fromObjOccId]" />
         <!-- Get the to object definition -->
         <xsl:variable name="fromObjDef" select="$objDefinitions[@ObjDef.ID = $fromObjOcc/@ObjDef.IdRef]" />
         <xsl:variable name="fromObjID" select="$fromObjDef/AttrDef[@AttrDef.Type='AT_ID']/AttrValue" />

         <!--
            If the object connected to will not be drawn as an activity then this transition should not be drawn.
         -->
         <xsl:variable name="isConnectedOccActivity">
            <xsl:call-template name="isActivity">
               <xsl:with-param name="objOcc" select="$toObjOcc" />
               <xsl:with-param name="objDef" select="$toObjDef" />
               <xsl:with-param name="allObjOccs" select="$objOccs" />
            </xsl:call-template>
         </xsl:variable>

         <xsl:if test="$isConnectedOccActivity = '&true;'">
            <xsl:variable name="cxnDefId" select="@CxnDef.IdRef" />

            <xpdl:Transition Id="{$objDef/CxnDef[@CxnDef.ID = $cxnDefId]/AttrDef[@AttrDef.Type = 'AT_ID']/AttrValue}"
					Name="{normalize-space($objDef/CxnDef[@CxnDef.ID = $cxnDefId]/AttrDef[@AttrDef.Type = 'AT_DESC']/AttrValue)}"
					From="{$fromObjID}" To="{$toObjID}">

               <!-- If this the source of this transition is an OT_RULE and not of operation AND (ie, it is OR or XOR) then this transition should be conditional -->
               <xsl:if test="$objDef/@TypeNum = 'OT_RULE' and not (starts-with($objDef/@SymbolNum, 'ST_OPR_AND')
							or starts-with($objDef/@SymbolNum, 'ST_AND'))">
					<xsl:variable name="ObjDef.IdRef" select="/AML/Group/Model/ObjOcc[@ObjOcc.ID=$toObjOccId]/@ObjDef.IdRef"/>
					<xsl:if test="(/AML/Group/Model/ObjOcc[@ObjOcc.ID=$thisObjOcc/@ObjOcc.ID]/@SymbolNum='ST_OPR_XOR_1' and /AML/Group/Model/ObjOcc[@ObjOcc.ID=$toObjOccId]/@SymbolNum='ST_EV') or
									(/AML/Group/Model/ObjOcc[@ObjOcc.ID=$thisObjOcc/@ObjOcc.ID]/@SymbolNum='ST_OPR_AND_1' and /AML/Group/Model/ObjOcc[@ObjOcc.ID=$toObjOccId]/@SymbolNum='ST_EV')">
	                  <xpdl:Condition Type="CONDITION">
						<xsl:value-of select="/AML/Group/ObjDef[@ObjDef.ID=$ObjDef.IdRef]/AttrDef[@AttrDef.Type='AT_NAME']/AttrValue"/>
					  </xpdl:Condition>
					</xsl:if>
               </xsl:if>

            </xpdl:Transition>
         </xsl:if>
      </xsl:if>

   </xsl:template>

   <!--
      ===================================================================================================================
      getFirstLinkedModel = Get any linked model of type EPC in the given linked ref.  A function object can reference multiple models of various types.
      This template will recover the first model reference that is of an EPC model.
      ===================================================================================================================
   -->
   <xsl:template name="getFirstLinkedModel">
      <xsl:param name="linkedRef" />
      <xsl:param name="models" />

      <!-- If there is a space in the linked ref then search for the first ref to a model -->
      <xsl:choose>
         <xsl:when test="contains ($linkedRef, ' ')">
            <xsl:variable name="link" select="substring-before ($linkedRef, ' ')" />
            <xsl:variable name="linkedModel" select="$models[@Model.ID = $link and starts-with (@Model.Type, 'MT_EEPC')]/@Model.ID" />

            <xsl:choose>
               <xsl:when test="$linkedModel">
                  <xsl:value-of select="$linkedModel/@Model.ID" />
               </xsl:when>
               <xsl:otherwise>
                  <xsl:call-template name="getFirstLinkedModel">
                     <xsl:with-param name="linkedRef" select="substring-after($linkedRef, ' ')" />
                     <xsl:with-param name="models" select="$models" />
                  </xsl:call-template>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:when>
         <xsl:otherwise>
            <xsl:value-of select="$models[@Model.ID = $linkedRef and starts-with (@Model.Type, 'MT_EEPC')]/@Model.ID" />
         </xsl:otherwise>
      </xsl:choose>

   </xsl:template>



   <!--
      ===================================================================================================================
      isActivity = this template will return 'true' if the given object will be drawn as an activity in the mode, else 'false' will be returned
      ===================================================================================================================
   -->
   <xsl:template name="isActivity">
      <xsl:param name="objOcc" />
      <xsl:param name="objDef" />
      <xsl:param name="allObjOccs" />

      <!-- If this is a function, rule or event type then it will be placed as an activity -->
      <xsl:choose>
         <xsl:when test="contains('&arisFunctions; &arisEvents; &arisRules;',$objDef/@TypeNum)">
            <xsl:text>&true;</xsl:text>
         </xsl:when>
         <xsl:otherwise>

            <xsl:choose>
               <xsl:when test="contains ('&arisParticipants;', $objDef/@TypeNum)">
                  <xsl:text>&false;</xsl:text>
               </xsl:when>
               <xsl:when test="contains ('&arisDataFields;', $objDef/@TypeNum)">

                  <xsl:text>&false;</xsl:text>
               </xsl:when>
               <!-- only 1 out-going connection -->
               <xsl:when test="count ($objOcc/CxnOcc) = 1 and count ($allObjOccs/CxnOcc[@ToObjOcc.ID = $objOcc/@ObjOcc.ID]) = 0">
                     <xsl:text>&false;</xsl:text>
               </xsl:when>
               <!-- Only 1 in-coming connection -->
               <xsl:when test="count ($objOcc/CxnOcc) = 0 and count ($allObjOccs/CxnOcc[@ToObjOcc.ID = $objOcc/@ObjOcc.ID]) = 1">
                     <xsl:text>&false;</xsl:text>
               </xsl:when>
               <!-- no connections -->
               <xsl:when test="count ($objOcc/CxnOcc) = 0 and count ($allObjOccs/CxnOcc[@ToObjOcc.ID = $objOcc/@ObjOcc.ID]) = 0">
                     <xsl:text>&false;</xsl:text>
               </xsl:when>
               <xsl:otherwise>
                     <xsl:text>&true;</xsl:text>
               </xsl:otherwise>
            </xsl:choose>

         </xsl:otherwise>
      </xsl:choose>

   </xsl:template>

   <!--
      ===================================================================================================================
      General template to copy all attributes and text of an element
      ===================================================================================================================
   -->
   <xsl:template match="@* | text()">

      <!-- Copy input attribute to output. -->
      <xsl:copy>
         <xsl:apply-templates select="@* | text()" />
      </xsl:copy>

   </xsl:template>

</xsl:stylesheet>

